unit Capitst1;
{See project file CAPITST.DPR for details										}
{Austin Kinsella Institute of Technology Carlow May 1999						}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Capi2;

type
  TMainForm = class(TForm)
    btnVersion: TButton;
    Label1: TLabel;
    lblCapiVer: TLabel;
    Label3: TLabel;
    lblManfVer: TLabel;
    btnManufacturer: TButton;
    lblManufacturer: TLabel;
    btnRegister: TButton;
    lblRegister: TLabel;
    btnDial: TButton;
    Label2: TLabel;
    edtCalledID: TEdit;
    Label4: TLabel;
    memLogBox: TMemo;
    btnProfile: TButton;
    Label5: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnVersionClick(Sender: TObject);
    procedure btnManufacturerClick(Sender: TObject);
    procedure btnRegisterClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnDialClick(Sender: TObject);
    procedure btnProfileClick(Sender: TObject);
  private
	Procedure Surprised (CapiSaid : CapiMessagePtr);
  	Procedure DoStage2;
	Procedure DoStage3 (NetMessage : CapiMessagePtr);
	Procedure DoStage5;
	Procedure DoStage6;
	Procedure DoStage7;
   Procedure DoStage9 (ConnectionID : DWord);
	Procedure DoStage10;
	Procedure DoStage13;
	Procedure DoStage14;
	Procedure DoStage15 (NetMessage : CapiMessagePtr);

  public
    { Public declarations }
  end;

var
	MainForm		: TMainForm;

implementation

{$R *.DFM}
Var
   PLCI			: DWord;

Procedure TMainForm.Surprised (CapiSaid : CapiMessagePtr);
{Called when command in message was unexpected - stick details in LogBox}
Var
	Step		: Integer;
   MessageHex	: String;
Begin {Surprised}
memLogBox.Lines.Add ('unexpected - ' +
       			   DecodeCommand (CapiSaid^.Header.Command) +  '_' +
                      DecodeSubCommand (CapiSaid^.Header.SubCommand));
MessageHex := '';
With CapiSaid^ do For Step := 0 to 20 do
	MessageHex := MessageHex + IntToHex (Body [Step], 2) + '-';
memLogBox.Lines.Add (MessageHex);
End; {Surprised}

procedure TMainForm.FormCreate(Sender: TObject);
{Check for installed card. If there activate buttons}
Var
	CapiError	: DWord;
   Serial		: CharArray8;
Begin {FormCreate}
CapiError := CAPI_INSTALLED;
If CapiError <> 0 then
	memLogBox.Lines.Add ('No ISDN card')
Else
	Begin {card present}
   memLogBox.Lines.Add ('ISDN card OK');
   CapiError := CAPI_GET_SERIAL_NUMBER (Serial);
   If Length (SErial) > 0 then
   	memLogBox.Lines.Add ('    serial - ' + Serial)
   Else
   	memLogBox.Lines.Add ('    no serial number');
   btnManufacturer.Enabled := TRUE;
   btnRegister.Enabled := TRUE;
   btnProfile.Enabled := TRUE;
	End; {card present}
End; {FormCreate}

procedure TMainForm.btnVersionClick(Sender: TObject);
{Call Get_Version, put results in CapiVer and ManfVer labels}
Var
   CapiMajorVer	: DWord;
	CapiMinorVer	: DWord;
   ManfMajorVer	: DWord;
   ManfMinorVer	: DWord;
   CapiError		: DWord;

begin {VersionClick}
CapiError := CAPI_GET_VERSION (CapiMajorVer, CapiMinorVer,
                               ManfMajorVer, ManfMinorVer);
lblCapiVer.Caption := IntToStr (CapiMajorVer) + '.' + IntToStr (CapiMinorVer);
lblManfVer.Caption := IntToStr (ManfMajorVer) + '.' + IntToStr (ManfMinorVer);
end; {VersionClick}

procedure TMainForm.btnManufacturerClick(Sender: TObject);
{Call Get_Manf, put the results in lblManufacturer}
Var
   TheName		: CharArray64;
begin {btnManufacturerClick}
CAPI_GET_MANUFACTURER (TheName);
lblManufacturer.Caption := TheName;
end; {btnManufacturerClick}

procedure TMainForm.btnRegisterClick(Sender: TObject);
{Register the application. Put the App ID in lblRegister}
{The returned ID is in Global Application ID}
Const
   MaxLogConnect	: DWord		= 3;		{data needed by Register in		}
   MaxBDataBlocks	: DWord		= 7;        {practice these would come from	}
   MaxBDataLen		: DWord		= 1024;     {some form of config			}
Var
	MessBuffSize	: DWord;
   CapiError		: DWord;
begin {btnRegisterClick}
MessBuffSize := 1024 + 1024 * MaxLogConnect;
CapiError := CAPI_REGISTER (MessBuffSize,
                       	MaxLogConnect,
                       	MaxBDataBlocks,
                       	MaxBDataLen,
                       	ApplicationID);
If CapiError = 0 then
	Begin {registration OK}
	lblRegister.Caption := IntToStr (ApplicationID);
   btnRegister.Enabled := FALSE;	{only reg once}
   btnDial.Enabled := TRUE;
   edtCalledID.Enabled := TRUE;	{let him enter number}
   End {registration OK}
Else
   memLogBox.Lines.Add ('Registration error - ' + IntToHex (CapiError, 4));
end; {btnRegisterClick}

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
{Say goodbye to DLL on the way out}
Var
	CapiError	: DWord;
begin {FormClose}
If (btnManufacturer.Enabled) AND NOT (btnRegister.Enabled)then
	Begin {are registered - sign off}
	CapiError := CAPI_RELEASE (ApplicationID);
	If CapiError <> 0 then
		ShowMessage ('Error on release: ' + IntToHex (CapiError, 4));
   End; {are registered - sign off}
end; {FormClose}

procedure TMainForm.btnDialClick(Sender: TObject);
{We are going through a call setup. Sequence of events is:					}
{App --->						<----CAPI                                   }
{ 1 CONNECT_REQ                                                            }
{ 2							CONNECT_CONF                                }
{ 3							CONNECT_ACTIVE_IND                          }
{ 4 CONNECT_ACTIVE_RESP                                                    }
{ 5 CONNECT_B3_REQ                                                         }
{ 6							CONNECT_B3_CONF                             }
{ 7							CONNECT_B3_ACTIVE_IND                       }
{ 8 CONNECT_B3_ACTIVE_RESP                                                 }
{             data transfer                                                }
{ 9 DISCONNECT_B3_REQ                                                      }
{10                            DISCONNECT_B3_CONF                          }
{11                            DISCONNECT_B3_IND                           }
{12 DISCONNECT_B3_RESP                                                     }
{13 DISCONNECT_REQ                                                         }
{14                            DISCONNECT_CONF                             }
{15                            DISCONNECT_IND                              }
{16 DISCONNECT_RESP                                                        }
{at stage 3 the network may reject the call,  with a DISCONNECT_IND.		}
{other errors are possible but we will only attempt the sequence as set out}
{and issue a disconnect as soon as the connection is complete				}
{Stage 1:                                                                  }
{Build the Header and Message in MessageText, then pass its address in		}
{MessagePtr to Put_Message													}
Var
	CapiError			: DWord;
   MessagePtr			: CapiMessagePtr;
   MessageText			: Array [0..40] of Char; {big enough to hold header + body}
   OffSet				: Integer;
   Controller			: DWord;
   Compatibility		: Word;
   CallTo				: String;
   EmptyStruct			: String;
begin {btnDialClick}
MessagePtr := @MessageText;
With MessagePtr^.Header do
	Begin {load the header}
   APPL_ID := ApplicationID;
   Command := cmd_CONNECT;
   SubCommand := CodeSubCommand (REQ);
   MessageNumber := 1;
   End; {load the header}
OffSet := 0;
{Parameters - in real life come from elsewhere}
Controller :=  1;
Compatibility := 0;
CallTo := Chr($80) + edtCalledID.Text; {x80 is suggested value of called party info}
EmptyStruct := '';
Offset := BuildMessage (MessagePtr, OffSet, Controller, parDWord);
Offset := BuildMessage (MessagePtr, OffSet, Compatibility, parWord);
Offset := BuildMessage (MessagePtr, OffSet, CallTo, parStruct);
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {calling party no}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {called party subaddress}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {calling party subaddress}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {B protocol}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {Bearer capability}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {Low-level compat}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {High-level compat}
Offset := BuildMessage (MessagePtr, OffSet, EmptyStruct, parStruct); {Additional info}
CapiError := CAPI_PUT_MESSAGE (ApplicationID, MessagePtr);
memLogBox.Lines.Add ('CONNECT_REQ issued');
If CapiError <> 0 then
	memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
Else
	DoStage2;
end; {btnDialClick}

Procedure TMainForm.DoStage2;
{Wait for & get connect conf, then wait for & get following message}
Var
	CapiError	: DWord;
   MessagePtr	: CapiMessagePtr;
Begin {DoStage2}
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);{assume connect conf}
memLogBox.Lines.Add ('CONNECT_CONF received');
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);
Case MessagePtr^.Header.Command of
	cmd_DISCONNECT		: DoStage15 (MessagePtr);
   cmd_CONNECT_ACTIVE	: DoStage3 (MessagePtr);
   else Surprised (MessagePtr);
   End; {Case Command}
End; {DoStage2}

Procedure TMainForm.DoStage3 (NetMessage : CapiMessagePtr);
{Got Connect Active IND. Respond (Stage4). If OK do stage 5}
Var
	OffSet			: Integer;
   ResponseText	: Array [1..30] of Char;
   Response		: CapiMessagePtr;
   CapiError		: Integer;
 {  NCPI			: String;
   NCCI			: DWord;
   Info			: Word; }
Begin {DoStage3}
memLogBox.Lines.Add ('CONNECT_ACTIVE_IND received');
{Retrieve the Physical Link Connection Identifier}
OffSet := 0;
OffSet := DecodeMessage (NetMessage, OffSet, PLCI, parDWord);
{Respond to connect active ind}
Response := @ResponseText;
With Response^.Header do
	Begin {load the header}
   APPL_ID := ApplicationID;
   Command := cmd_CONNECT_ACTIVE;
   SubCommand := CodeSubCommand (RESP);
   MessageNumber := NetMessage^.Header.MessageNumber; {use number from ind}
   End; {load the header}
OffSet := 0;
Offset := BuildMessage (Response, OffSet, PLCI, parDWord);
CapiError := CAPI_PUT_MESSAGE (ApplicationID, Response);
memLogBox.Lines.Add ('CONNECT_ACTIVE_RESP issued');
If CapiError <> 0 then
	memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
Else
	DoStage5;
End; {DoStage3}

Procedure TMainForm.DoStage5;
{Now request B channel}
Var
   ResponseText	: Array [1..30] of Char;
   Response		: CapiMessagePtr;
   CapiError		: Integer;
   NCPI			: String;
   OffSet			: Integer;
Begin {DoStage5}
{Now request the B channel}
NCPI := ''; {Network control protocol information  - transparent}
Response := @ResponseText;
With Response^.Header do
	Begin {load the header}
   APPL_ID := ApplicationID;
   Command := cmd_CONNECT_B3;
   SubCommand := CodeSubCommand (REQ);
   MessageNumber := 2;
   End; {load the header}
OffSet := 0;
Offset := BuildMessage (Response, OffSet, PLCI, parDWord);
Offset := BuildMessage (Response, OffSet, NCPI, parStruct);
CapiError := CAPI_PUT_MESSAGE (ApplicationID, Response);
memLogBox.Lines.Add ('CONNECT_B3_REQ issued');
If CapiError <> 0 then
	memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
Else
	DoStage6;
End; {DoStage5}

Procedure TMainForm.DoStage6;
{Wait for CONF to connect B3 request - if OK, go stage7 to wait ind}
Var
	CapiError	: DWord;
   MessagePtr	: CapiMessagePtr;
Begin {DoStage6}
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);
If MessagePtr^.Header.Command = cmd_CONNECT_B3 then
	Begin {conf received}
   memLogBox.Lines.Add ('CONNECT_B3_CONF received');
   DoStage7;
   End {conf received}
else
	Surprised (MessagePtr);
End; {DoStage6}

Procedure TMainForm.DoStage7;
{Wait for IND. If received, issue response then start disconnect }
Var
	CapiError		: DWord;
   MessagePtr		: CapiMessagePtr;
   ResponseText	: Array [1..30] of Char;
   Response		: CapiMessagePtr;
   NCCI			: DWord;
   OffSet			: Integer;
Begin {DoStage7}
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);
 If MessagePtr^.Header.Command = cmd_CONNECT_B3_ACTIVE then
	Begin {IND received}
   memLogBox.Lines.Add ('CONNECT_B3_ACTIVE_IND received');
   OffSet := 0;
   OffSet := DecodeMessage (MessagePtr, OffSet, NCCI, parDWord);
	Response := @ResponseText;
	With Response^.Header do
   	Begin {load the header}
   	APPL_ID := ApplicationID;
   	Command := cmd_CONNECT_B3_ACTIVE;
   	SubCommand := CodeSubCommand (RESP);
   	MessageNumber := MessagePtr^.Header.MessageNumber; {use number from ind}
   	End; {load the header}
	OffSet := 0;
	Offset := BuildMessage (Response, OffSet, NCCI, parDWord);
	CapiError := CAPI_PUT_MESSAGE (ApplicationID, Response);
	memLogBox.Lines.Add ('CONNECT_B3_ACTIVE_RESP issued');
	If CapiError <> 0 then
		memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
	Else
		DoStage9 (NCCI);
   End {conf received}
else
	Surprised (MessagePtr);
End; {DoStage7}

Procedure TMainForm.DoStage9 (ConnectionID : DWord);
{Disconnect B channel}
Var
   RequestText		: Array [1..30] of Char;
   Request			: CapiMessagePtr;
   CapiError		: Integer;
   NCPI			: String;
   OffSet			: Integer;
Begin {DoStage9}
NCPI := ''; {Network control protocol information  - none}
Request := @RequestText;
With Request^.Header do
	Begin {load the header}
   APPL_ID := ApplicationID;
   Command := cmd_DISCONNECT_B3;
   SubCommand := CodeSubCommand (REQ);
   MessageNumber := 3;
   End; {load the header}
OffSet := 0;
Offset := BuildMessage (Request, OffSet, ConnectionID, parDWord);
Offset := BuildMessage (Request, OffSet, NCPI, parStruct);
CapiError := CAPI_PUT_MESSAGE (ApplicationID, Request);
memLogBox.Lines.Add ('DISCONNECT_B3_REQ issued');
If CapiError <> 0 then
	memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
Else
	DoStage10;
End; {DoStage9}

Procedure TMainForm.DoStage10;
{Wait for & get conf, then wait for & get ind}
Var
	CapiError		: DWord;
   MessagePtr		: CapiMessagePtr;
   NCCI			: DWord;
   ResponseText	: Array [1..30] of Char;
   Response		: CapiMessagePtr;
   OffSet			: Integer;
Begin {DoStage10}
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);{assume connect conf}
memLogBox.Lines.Add ('DISCONNECT_B3_CONF received');
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);
If MessagePtr^.Header.Command = cmd_DISCONNECT_B3 then
   Begin {got B3 ind}
	memLogBox.Lines.Add ('DISCONNECT_B3_IND received');
   OffSet := 0;
   OffSet := DecodeMessage (MessagePtr, OffSet, NCCI, parDWord);
	Response := @ResponseText;
	With Response^.Header do
   	Begin {load the header}
   	APPL_ID := ApplicationID;
   	Command := cmd_DISCONNECT_B3;
   	SubCommand := CodeSubCommand (RESP);
   	MessageNumber := MessagePtr^.Header.MessageNumber; {use number from ind}
   	End; {load the header}
	OffSet := 0;
	Offset := BuildMessage (Response, OffSet, NCCI, parDWord);
	CapiError := CAPI_PUT_MESSAGE (ApplicationID, Response);
	memLogBox.Lines.Add ('DISCONNECT_B3_RESP issued');
	If CapiError <> 0 then
		memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
	Else
		DoStage13;
   End {got B3 ind}
else Surprised (MessagePtr);
End; {DoStage10}

Procedure TMainForm.DoStage13;
{Disconnect}
Var
   RequestText		: Array [1..30] of Char;
   Request			: CapiMessagePtr;
   CapiError		: Integer;
   OffSet			: Integer;
   AddInfo			: String;
Begin {DoStage13}
AddInfo := ''; {no additional info}
Request := @RequestText;
With Request^.Header do
	Begin {load the header}
   APPL_ID := ApplicationID;
   Command := cmd_DISCONNECT;
   SubCommand := CodeSubCommand (REQ);
   MessageNumber := 4;
   End; {load the header}
OffSet := 0;
Offset := BuildMessage (Request, OffSet, PLCI, parDWord);
Offset := BuildMessage (Request, OffSet, AddInfo, parStruct);
CapiError := CAPI_PUT_MESSAGE (ApplicationID, Request);
memLogBox.Lines.Add ('DISCONNECT_REQ issued');
If CapiError <> 0 then
	memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4))
Else
	DoStage14;
End; {DoStage13}

Procedure TMainForm.DoStage14;
{Wait for & get conf, then wait for & get ind}
Var
	CapiError		: DWord;
   MessagePtr		: CapiMessagePtr;
Begin {DoStage14}
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);{assume conf}
memLogBox.Lines.Add ('DISCONNECT_CONF received');
CapiError := CAPI_WAIT_FOR_SIGNAL (ApplicationID);
CapiError := CAPI_GET_MESSAGE (ApplicationID, MessagePtr);
If MessagePtr^.Header.Command = cmd_DISCONNECT then
   Begin {got B3 ind}
	memLogBox.Lines.Add ('DISCONNECT_IND received');
   DoStage15 (MessagePtr);
   End {got B3 ind}
else Surprised (MessagePtr);
End; {DoStage14}

Procedure TMainForm.DoStage15 (NetMessage : CapiMessagePtr);
{Disconnect IND}
{Decode reason, build and send RESP}
Var
	OffSet			: Integer;
   PLCI			: DWord;
   Reason			: Word;
   ResponseText	: Array [1..30] of Char;
   Response		: CapiMessagePtr;
   CapiError		: Integer;
Begin {NetDisconnect}
OffSet := 0;
OffSet := DecodeMessage (NetMessage, OffSet, PLCI, parDWord);
OffSet := DecodeMessage (NetMessage, OffSet, Reason, parWord);
memLogBox.Lines.Add ('DISCONNECT_IND received - ' + IntToHex (Reason, 4));
{build and send response}
Response := @ResponseText;
With Response^.Header do
	Begin {load the header}
   APPL_ID := ApplicationID;
   Command := cmd_DISCONNECT;
   SubCommand := CodeSubCommand (RESP);
   MessageNumber := NetMessage^.Header.MessageNumber; {use number from ind};
   End; {load the header}
OffSet := 0;
Offset := BuildMessage (Response, OffSet, PLCI, parDWord);
CapiError := CAPI_PUT_MESSAGE (ApplicationID, Response);
If CapiError <> 0 then
	memLogBox.Lines.Add ('Put Message error - ' + IntToHex (CapiError, 4));
End; {DoStage15}


procedure TMainForm.btnProfileClick(Sender: TObject);
{Call Get_Profile, display results in LogBox}
Var
   Control		: DWord;
	CapiError	: DWord;
   Profile		: ByteArray64;
Function IntToYN (Value : Byte; BitNo : Byte) : String;
{Return Yes if bit BitNo is set in Value, else No. Bitno range 0..7}
Begin {IntToYN}
If (Value AND ( 1 shl BitNo)) <> 0 then
	Result := 'Yes'
Else
	Result := 'No';
End; {IntToYN}
begin {btnProfileClick}
Control := 1;
CapiError := CAPI_GET_PROFILE (Profile, Control);
If CapiError <> 0 then
	memLogBox.Lines.Add ('Error getting profile: ' + IntToHex (CapiError, 4))
Else
	Begin {card present}
   memLogBox.Lines.Add ('Profile:');
   memLogBox.Lines.Add ('    Controllers: ' + IntToStr (Profile [0])); {2 bytes, ignore high}
   memLogBox.Lines.Add ('    B channels: ' + IntToStr (Profile [2])); {2 bytes, ignore high}
   memLogBox.Lines.Add ('    Global options: '); {4 bytes, only low used}
   memLogBox.Lines.Add ('        internal controller: ' + IntToYN (Profile [4], 0));
   memLogBox.Lines.Add ('        external controller: ' + IntToYN (Profile [4], 1));
   memLogBox.Lines.Add ('        handset support: ' + IntToYN (Profile [4], 2));
   memLogBox.Lines.Add ('        DTMF support: ' + IntToYN (Profile [4], 3));
   memLogBox.Lines.Add ('        supplementary services: ' + IntToYN (Profile [4], 4));
   memLogBox.Lines.Add ('        channel alloc support: ' + IntToYN (Profile [4], 5));
   memLogBox.Lines.Add ('    B1 protocol: ');  {4 bytes, 1 & 2 used}
   memLogBox.Lines.Add ('        64k, HDLC framing: ' + IntToYN (Profile [8], 0));
   memLogBox.Lines.Add ('        64k, bit transparent: ' + IntToYN (Profile [8], 1));
   memLogBox.Lines.Add ('        V.110 async: ' + IntToYN (Profile [8], 2));
   memLogBox.Lines.Add ('        V.110 sync, HDLC framing: ' + IntToYN (Profile [8], 3));
   memLogBox.Lines.Add ('        T.30 modem, fax group 3: ' + IntToYN (Profile [8], 4));
   memLogBox.Lines.Add ('        64k inverted, HDLC framing: ' + IntToYN (Profile [8], 5));
   memLogBox.Lines.Add ('        56k, bit transparent: ' + IntToYN (Profile [8], 6));
   memLogBox.Lines.Add ('        Modem with all negotiations: ' + IntToYN (Profile [8], 7));
   memLogBox.Lines.Add ('        Modem async operation: ' + IntToYN (Profile [9], 0));
   memLogBox.Lines.Add ('        Modem sync, HDLC framing: ' + IntToYN (Profile [9], 1));
   memLogBox.Lines.Add ('    B2 protocol: ');  {4 bytes, 1 & 2 used}
   memLogBox.Lines.Add ('        ISO 7776 - X.75 SLP: ' + IntToYN (Profile [12], 0));
   memLogBox.Lines.Add ('        transparent: ' + IntToYN (Profile [12], 1));
   memLogBox.Lines.Add ('        SDLC: ' + IntToYN (Profile [12], 2));
   memLogBox.Lines.Add ('        LAPD to Q.921: ' + IntToYN (Profile [12], 3));
   memLogBox.Lines.Add ('        T.30 for fax group 3: ' + IntToYN (Profile [12], 4));
   memLogBox.Lines.Add ('        PPP protocol: ' + IntToYN (Profile [12], 5));
   memLogBox.Lines.Add ('        transparent, B1 framing error ignored: ' + IntToYN (Profile [12], 6));
   memLogBox.Lines.Add ('        Modem error correction & compression (V.42 bis): ' + IntToYN (Profile [12], 7));
   memLogBox.Lines.Add ('        ISO 7776 - X.75 SLP with V.42 bis: ' + IntToYN (Profile [13], 0));
   memLogBox.Lines.Add ('        V.120 async: ' + IntToYN (Profile [13], 1));
   memLogBox.Lines.Add ('        V.120 async with V.42 bis: ' + IntToYN (Profile [13], 2));
   memLogBox.Lines.Add ('        V.120 bit-transparent: ' + IntToYN (Profile [13], 3));
   memLogBox.Lines.Add ('        V.120 LAPD to Q.921 with free SAPI selection: ' + IntToYN (Profile [13], 4));
   memLogBox.Lines.Add ('    B3 protocol: ');  {4 bytes, low used}
   memLogBox.Lines.Add ('        transparent: ' + IntToYN (Profile [16], 0));
   memLogBox.Lines.Add ('        T.90L: ' + IntToYN (Profile [16], 1));
   memLogBox.Lines.Add ('        ISO 8208 (X.25 DTE-DTE): ' + IntToYN (Profile [16], 2));
   memLogBox.Lines.Add ('        X.25 DCE: ' + IntToYN (Profile [16], 3));
   memLogBox.Lines.Add ('        T.30 fax group 3: ' + IntToYN (Profile [16], 4));
   memLogBox.Lines.Add ('        T.30 fax group 3 with ext: ' + IntToYN (Profile [16], 5));
   memLogBox.Lines.Add ('        Modem: ' + IntToYN (Profile [16], 7));
   End; {Card present}
end; {btnProfileClick}

end. {CapiTst1}

